#include <stdio.h>
#include <math.h>

#define TOL 0.00000000001
#define MEL 100

int NEL;

double coeffs[MEL][MEL];
double consts[MEL];
char rowtaken[MEL];
int solvedrow[MEL];

void Gauss()
{
	double best;
	int i, j, k, index;

	memset( solvedrow, -1, sizeof(solvedrow) );
	memset( rowtaken, 0, sizeof(rowtaken) );

	for (i=0; i < NEL; i ++) {
		best = 0.0;

		for (j=0; j < NEL; j ++) {
			if (!rowtaken[j] && fabs(coeffs[j][i]) > best) {
				index = j;
				best = fabs( coeffs[j][i] );
			}
		}
		if (best < TOL) continue;
		for (j=0; j < NEL; j ++) {
			if (j != index) {
				best = coeffs[j][i] / coeffs[index][i];
				for (k=0; k < NEL; k ++) {
					coeffs[j][k] -= coeffs[index][k] * best;
				}
				consts[j] -= consts[index] * best;
			}
		}
		solvedrow[i] = index;
		rowtaken[index] = 1;
	}
}

int main()
{
	int n, m, o, d;
	scanf("%d %d", &n, &m);
	NEL = n*m;

	memset( coeffs, 0, sizeof(coeffs) );
	memset( consts, 0, sizeof(consts) );

	for (n=0; n < NEL-6; n ++) {
		coeffs[n][n] = 1.0;
		coeffs[n][n+1] = -1.0/6.0;
		coeffs[n][n+2] = -1.0/6.0;
		coeffs[n][n+3] = -1.0/6.0;
		coeffs[n][n+4] = -1.0/6.0;
		coeffs[n][n+5] = -1.0/6.0;
		coeffs[n][n+6] = -1.0/6.0;
		consts[n] = 1;
	}
	for (; n < NEL-1; n ++) {
		coeffs[n][n] = 1.0 - (6.0-(NEL-1-n))/6.0;
		for (m=n+1; m < NEL; m ++) {
			coeffs[n][m] = -1.0/6.0;
		}
		consts[n] = 1;
	}
	coeffs[NEL-1][NEL-1] = 1.0;
	consts[NEL-1] = 0.0;

	while (scanf("%d %d", &o, &d) == 2) {
		o -= 1;
		d -= 1;
		for (n=0; n < NEL; n ++) {
			coeffs[o][n] = 0.0;
		}
		coeffs[o][o] = 1.0;
		coeffs[o][d] = -1.0;
		consts[o] = 0.0;
	}

#ifdef DEBUG
	for (n=0; n < NEL; n ++) {
		for (m=0; m < NEL; m ++) {
			printf("%7.3lf ", coeffs[n][m]);
		}
		printf(" = %7.3lf\n", consts[n]);
	}
#endif
	Gauss();

#ifdef DEBUG
	printf("\n");
	for (n=0; n < NEL; n ++) {
		for (m=0; m < NEL; m ++) {
			printf("%7.3lf ", coeffs[n][m]);
		}
		printf(" = %7.3lf\n", consts[n]);
	}
#endif

	n = solvedrow[0];
	printf("%.2lf\n", consts[n] / coeffs[n][0] );
}
