#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#define MAX_NODES 450

typedef struct {
  int deg;           /* number of outgoing edges */
  int adj[MAX_NODES];
  
  /* the following is not necessary, but useful in many situations */
  int cost[MAX_NODES];
} Node;

int M, N;
int squares[21][21];

void BFS_shortest_path(Node graph[MAX_NODES], int n, int src, int D[MAX_NODES],
		       int P[MAX_NODES])
{
  char used[MAX_NODES];
  int queue[MAX_NODES+1];
  int q_in, q_out, i, j;
  int v, w;

  memset(used, 0, MAX_NODES);
  queue[0] = src;
  used[src] = 1;
  q_in = 1;
  q_out = 0;
  
  for (i = 0; i < MAX_NODES; i++) {
    D[i] = -1;
    P[i] = -1;
  }
  D[src] = 0;

  while (q_out < q_in) {
    v = queue[q_out++];
    for (i = 0; i < graph[v].deg; i++) {
      w = graph[v].adj[i];
      if (!used[w]) {
	D[w] = D[v] + 1;
	P[w] = v;
	queue[q_in++] = w;
	used[w] = 1;
      } else if (D[v] + 1 == D[w]) {
	/* put tie-breaker here */
	/* eg. find largest path in lexicographic order */
	P[w] = P[w] > v ? P[w] : v;
      }
    }
  }
}

void clear(Node graph[MAX_NODES], int n)
{
  int i;
  for (i = 0; i < n; i++) {
    graph[i].deg = 0;
  }
}

void add_edge(Node graph[MAX_NODES], int v, int w, int cost)
{
  int i;

  /* make sure that we have no duplicate edges */
  for (i = 0; i < graph[v].deg; i++) {
    if (graph[v].adj[i] == w) {
      assert(0);
    }
  }

  graph[v].cost[graph[v].deg] = cost;
  graph[v].adj[graph[v].deg] = w;
  graph[v].deg++;
}

int theIndex( int i, int j )
{
  return i * ( N+1 ) + j;
}

void rowCol( int index, int* row, int* col )
{
  *row = index / (N+1);
  *col = index % (N+1);
}

int canFitIn( int i, int j )
{
  if ( i > 0 )
    {
      if ( j > 0 )
	{
	  if ( squares[i-1][j-1] == 1 ) return 0;
	}
      if ( j < M )
	{
	  if ( squares[i-1][j] == 1 ) return 0;
	}
    }
  if ( i < M )
    {
      if ( j > 0 )
	{
	  if ( squares[i][j-1] == 1 ) return 0;
	}
      if ( j < M )
	{
	  if ( squares[i][j] == 1 ) return 0;
	}
    }

  return 1;
}

void computeNeighbours( Node graph[MAX_NODES], int i, int j )
{
  int ind = theIndex( i, j );
  graph[ind].deg = 0;
  if ( i > 0 )
    {
      if ( canFitIn( i-1, j ) )
	{
	  add_edge( graph, ind, theIndex( i-1, j ), 1 );
	}
    }
  if ( i < M )
    {
      if ( canFitIn( i+1, j ) )
	{
	  add_edge( graph, ind, theIndex( i+1, j ), 1 );
	}
    }
  if ( j > 0 )
    {
      if ( canFitIn( i, j-1 ) )
	{
	  add_edge( graph, ind, theIndex( i, j-1 ), 1 );
	}
    }
  if ( j < N )
    {
      if ( canFitIn( i, j+1 ) )
	{
	  add_edge( graph, ind, theIndex( i, j+1 ), 1 );
	}
    } 
}

int main()
{
  Node graph[MAX_NODES];
  int D[MAX_NODES];
  int P[MAX_NODES];
  int i, j, fromX, fromY, toX, toY;

  while( (scanf( "%d%d", &M, &N ) == 2) && !((M == 0) && (N == 0)))
    {
      for (i = 0; i < M; i++ )
	{
	  for ( j = 0; j < N; j++ )
	    {
	      scanf( "%d", &squares[i][j] );
	    }
	}

      for ( i = 0; i <= M; i++ )
	{
	  for ( j = 0; j <= N; j++ )
	    {
	      computeNeighbours( graph, i, j );
	    }
	}

      scanf( "%d%d%d%d", &fromX, &fromY, &toX, &toY );
      BFS_shortest_path( graph, (M+1)*(N+1), theIndex( fromX, fromY ), D, P );
      printf( "%d\n", D[theIndex(toX, toY)] );
    }
}
