#include <math.h>
#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#define	inpfpath		"fence.inp"
#define	outpfpath		"fence.out"

#define TWR_C_MAX		1000
#define	Pi			3.1415925359

void sys_err (const char *format, ...)
{
 	va_list			va;

        va_start (va, format);
        fprintf (stderr, "** ERROR **:\n\t");
        vfprintf (stderr, format, va);
        fprintf (stderr, ".\n\tLast system error: %i %s.\n\n",
                 errno, strerror (errno));
        va_end (va);
}

typedef struct Twr_ts		Twr;

struct Twr_ts {
	double			X,
				Y,
				R;
};

Twr             towers [TWR_C_MAX];
Twr		twr_l [(TWR_C_MAX+1)/2];
Twr		twr_r [(TWR_C_MAX+1)/2];
unsigned int	n, twr_l_c, twr_r_c;
double		sol;

int twycmpfn (const void *a, const void *b)
{
	Twr		*t1, *t2;

	t1 = (Twr*) a;
	t2 = (Twr*) b;
	if ( t1->Y > t2->Y ) {
		return (1);
	} else if ( t1->Y < t2->Y ) {
		return (-1);
	} else {
		return (0);
	}
}

#define	sqr(x)			( (x)*(x) )
#define	sgn(x)			( (x) > 0 ? (1) : (-1) )

double phi (double x1, double y1, double r1, double x2, double y2, double r2)
{
	double		dx, dy, dr;
	double		phi, psi0, psi;

	dx = x2 - x1;
	dy = y2 - y1;
	dr = r2 - r1;
	psi = atan2 (dr, sqrt (sqr (dx) + sqr (dy)));
	psi0 = atan2 (dy, dx);
	phi = psi0 - psi;
	if ( phi < -Pi ) {
		phi += Pi;
	} else if ( phi > Pi ) {
		phi = Pi - phi;
	}
	return (phi);
}

double alpha (double x1, double y1, double x2, double y2)
{
	double		p1, p2;

	p1 = atan2 (y1, x1);
	p2 = atan2 (y2, x2);
	if ( p1 > 0 ) {
		if ( p2 < p1 && p2 > 0 ) {
			return (p1 - p2);
		} else {
			return (p2 - p1);
		}
	} else {
		if ( p2 > p1 && p2 < 0 ) {
			return (p2 - p1);
		} else {
			return (p1 - p2);
		}
	}
}

double cnv (double p1, double p2)
{
	if ( p1 > 0 ) {
		if ( p2 < p1 && p2 > 0 ) {
			return (p1 - p2);
		} else {
			return (p2 - p1);
		}
	} else {
		if ( p2 > p1 && p2 < 0 ) {
			return (p2 - p1);
		} else {
			return (p1 - p2);
		}
	}
}

double
l
(double x1, double y1, double r1,
 double x2, double y2, double r2,
 double x3, double y3, double r3)
{
	double			phi1, phi2;

	phi1 = phi (x1, y1, r1, x2, y2, r2);
	phi2 = -phi (x2, y2, r2, x3, y3, r3);
	printf ("%lf\n", fabs (cnv (phi2, phi1))/Pi);
	return (fabs (cnv (phi2, phi1)) * r2);
}

#define	delta(x1,y1,r1,x2,y2,r2)	\
		(\
			sqrt (\
				sqr (r2-r1) +\
				sqr (x2-x1) +\
				sqr (y2-y1)\
			)\
		)

void calc_sol (void)
{
	unsigned int	c;

	for ( c = 1; c < n; c++ ) {
		sol += delta (
			towers [c-1].X, towers [c-1].Y, towers [c-1].R,
			towers [c].X, towers [c].Y, towers [c].R
		);
	}
	sol += delta (
			towers [n-1].X, towers [n-1].Y, towers [n-1].R,
			towers [0].X, towers [0].Y, towers [0].R
		);
	for ( c = 2; c < n; c++ ) {
		sol += l (
			towers [c-2].X, towers [c-2].Y, towers [c-2].R,
			towers [c-1].X, towers [c-1].Y, towers [c-1].R,
			towers [c-0].X, towers [c-0].Y, towers [c-0].R
		);
	}
	sol += l (
			towers [n-2].X, towers [n-2].Y, towers [n-2].R,
			towers [n-1].X, towers [n-1].Y, towers [n-1].R,
			towers [0].X, towers [0].Y, towers [0].R
	);
	sol += l (
			towers [n-1].X, towers [n-1].Y, towers [n-1].R,
			towers [0].X, towers [0].Y, towers [0].R,
			towers [1].X, towers [1].Y, towers [1].R
		);
}

int go (void)
{
	double		xmean;
	unsigned int	c;

	sol = 0;
	if ( n == 0 ) {
		return (0);
	}
	if ( n == 1 ) {
		sol = 2.0*Pi*towers [0].R;
		return (0);
	}
	if ( n == 2 ) {
		sol = 2.0*delta (
			towers [0].X, towers [0].Y, towers [0].R,
			towers [1].X, towers [1].Y, towers [1].R
		);
		sol += l (
			towers [0].X, towers [0].Y, towers [0].R,
			towers [1].X, towers [1].Y, towers [1].R,
			towers [0].X, towers [0].Y, towers [1].R
		);
		sol += l (
			towers [1].X, towers [1].Y, towers [1].R,
			towers [0].X, towers [0].Y, towers [0].R,
			towers [1].X, towers [1].Y, towers [1].R
		);
		return (0);
	}
	xmean = 0;
	for ( c = 0; c < n; c++ ) {
		xmean += towers [c].X;
	}
	xmean /= n;
	twr_l_c = 0;
	twr_r_c = 0;
	for ( c = 0; c < n; c++ ) {
		if ( towers [c].X < xmean ) {
			twr_l [twr_l_c].X = towers[c].X;
			twr_l [twr_l_c].Y = towers[c].Y;
			twr_l [twr_l_c].R = towers[c].R;
			twr_l_c++;
		} else {
			twr_r [twr_r_c].X = towers [c].X;
			twr_r [twr_r_c].Y = towers [c].Y;
			twr_r [twr_r_c].R = towers [c].R;
			twr_r_c++;
		}
	}
	qsort (twr_l, twr_l_c, sizeof (twr_l [0]), twycmpfn);
	qsort (twr_r, twr_r_c, sizeof (twr_r [0]), twycmpfn);
	for ( c = 0; c < twr_r_c; c++ ) {
		towers [c].X = twr_r [c].X;
		towers [c].Y = twr_r [c].Y;
		towers [c].R = twr_r [c].R;
	}
	for ( c = 0; c < twr_l_c; c++ ) {
		towers [c+twr_r_c].X = twr_l [twr_l_c - c - 1].X;
		towers [c+twr_r_c].Y = twr_l [twr_l_c - c - 1].Y;
		towers [c+twr_r_c].R = twr_l [twr_l_c - c - 1].R;
	}
	calc_sol ();
	return (0);
}

int init (void)
{
	FILE			*inpf;

	errno = 0;
        if ( (inpf = fopen (inpfpath, "r")) == NULL ) {
        	sys_err ("Unabe to open the input file ('%s')", inpfpath);
                return (-1);
	}
	n = 0;
	while ( !feof (inpf) ) {
		if ( 	fscanf (inpf, "%lf %lf %lf\n",
				&towers [n].X, &towers [n].Y, &towers [n].R)
			!= 3
		) {
			sys_err ("Error inputting tower number %u", n+1);
			fclose (inpf);
			return (-1);
		}
		n++;
	}
	if ( fclose (inpf) ) {
        	sys_err ("Unable to close the output file ('%s')",
                         outpfpath);
                return (-1);
        }
	return (0);
}

int shut (void)
{
	FILE		*outpf;

        if ( (outpf = fopen (outpfpath, "w")) == NULL ) {
        	sys_err ("Unable to open the output file ('%s')", outpfpath);
                return (-1);
	}
	fprintf (outpf, "%.2lf\n", sol);
	if ( ferror (outpf) ) {
          	sys_err ("Error writing to the output file ('%s')",
                         outpfpath);
               	fclose (outpf);
                return (-1);
        }
	if ( fclose (outpf) ) {
        	sys_err ("Unable to close the output file ('%s')", outpfpath);
                return (-1);
        }
	return (0);
}

int main (void)
{
 	if ( init () ) {
        	return (1);
        }
	if ( go () ) {
        	return (1);
 	}
	if ( shut () ) {
        	return (1);
	}
        return (0);
}
