#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#define NO_STDERR_OUTP

#define inpf_path               ""
#define outpf_path              ""

/* Messaging functionality: */
#ifdef  NO_STDERR_OUTP

void m_sys_err (const char *format, ...)
{
         if ( format ) {
            /* We don't need "format". */
         }
}

void m_err (const char *format, ...)
{
        if ( format ) {
           /* We don't need "format". */
        }
}

#else /* NO_STDERR_OUTP */

void m_sys_err (const char *format, ...)
{
         va_list                 va;
         int                     o_errno;

         o_errno = errno;
         va_start (va, format);
         fprintf (stderr, "Error:\n\t");
         vfprintf (stderr, format, va);
         fprintf (stderr, "\n\tLast system error: %i %s.\n",
                          o_errno, strerror (o_errno));
         va_end (va);
}

void m_err (const char *format, ...)
{
        va_list                 va;

        va_start (va, format);
        fprintf (stderr, "Error:\n\t");
        vfprintf (stderr, format, va);
        fprintf (stderr, "\n");
        va_end (va);
}

#endif /* NO_STDERR_OUTP */

/* Solution: */
#define      CARDS_C_MAX        10000

typedef      int                card_t [4];

card_t                          cards [CARDS_C_MAX];
unsigned int                    cards_c, max_cards_c;

int are_a_set (unsigned int a, unsigned int b, unsigned int c)
{
        if ( cards [a][0] == cards [b][0] && cards [a][0] == cards [c][0] &&
             cards [a][1] == cards [b][1] && cards [a][1] == cards [c][1] &&
             cards [a][2] == cards [b][2] && cards [a][2] == cards [c][2] &&
             cards [a][3] == cards [b][3] && cards [a][3] == cards [c][3]
        ) {
          return (1);
        }
        if ( cards [a][0] != cards [b][0] &&
             cards [a][0] != cards [c][0] &&
             cards [b][0] != cards [c][0] &&
             cards [a][1] != cards [b][1] &&
             cards [a][1] != cards [c][1] &&
             cards [b][1] != cards [c][1] &&
             cards [a][2] != cards [b][2] &&
             cards [a][2] != cards [c][2] &&
             cards [b][2] != cards [c][2] &&
             cards [a][3] != cards [b][3] &&
             cards [a][3] != cards [c][3] &&
             cards [b][3] != cards [c][3]
        ) {
          return (1);
        }
        return (0);
}

int are_sets (void)
{
        unsigned int            i, j;

        if ( cards_c < 3 ) {
           return (0);
        }
        for ( i = 0; i < cards_c-1; i++ ) {
            for ( j = 0; j < i; j++ ) {
                if ( are_a_set (cards_c-1, i, j) ) {
                   return (1);
                }
            }
        }
        return (0);
}

int card_exists (int a, int b, int c, int d)
{
        unsigned int            i;

        for ( i = 0; i < cards_c; i++ ) {
            if ( cards [i][0] == a &&
                 cards [i][1] == b &&
                 cards [i][2] == c &&
                 cards [i][3] == d
            ) {
               return (1);
            }
        }
        return (0);
}

void print_cards (void)
{
        unsigned int            c;

        printf ("Better soulution:\n");
        printf (" N   A B C D\n");
        for ( c = 0; c < cards_c; c++ ) {
            printf ("%3u %i %i %i %i\n",
                   c+1,
                   cards [c][0], cards [c][1], cards [c][2], cards [c][3]);
        }
}

void btrack (void)
{
        int                     a, b, c, d;

        if ( cards_c > max_cards_c ) {
           max_cards_c = cards_c;
           print_cards ();
        }
        for ( a = cards [cards_c-1][0]; a < 3; a++ ) {
            for ( b = cards [cards_c-1][1]; b < 3; b++ ) {
                for ( c = cards [cards_c-1][2]; c < 3; c++ ) {
                    for ( d = cards [cards_c-1][3]; d < 3; d++ ) {
                        if ( card_exists (a, b, c, d) ) {
                           continue;
                        }
                        cards [cards_c][0] = a;
                        cards [cards_c][1] = b;
                        cards [cards_c][2] = c;
                        cards [cards_c][3] = d;
                        cards_c++;
                        if ( !are_sets () ) {
                           btrack ();
                        }
                        cards_c--;
                    }
                }
            }
        }
}

int go (void)
{
        max_cards_c = 0;
        cards_c = 1;
        cards [0][0] = 0;
        cards [0][1] = 0;
        cards [0][2] = 0;
        cards [0][3] = 0;
        btrack ();
        printf ("%u\n", max_cards_c);
        return (0);
}

int init (void)
{
        FILE                    *inpf;

        if ( inpf_path [0] == '\0' ) {
           inpf = stdin;
        } else {
           if ( (inpf = fopen (inpf_path, "r")) == NULL ) {
                  m_sys_err ("Unable to open input (%s)", inpf_path);
                  return (-1);
           }
        }
        if ( fclose (inpf) ) {
           m_sys_err ("Unable to close input (%s)", inpf_path);
           return (-1);
        }
        return (0);
}

int shut (void)
{
        FILE                    *outpf;

        if ( outpf_path [0] == '\0' ) {
           outpf = stdout;
        } else {
                if ( (outpf = fopen (outpf_path, "w")) == NULL ) {
                   m_sys_err ("Unable to open output (%s)", outpf_path);
                   return (-1);
                }
        }
        if ( ferror (outpf) ) {
           m_sys_err ("Error writing output to %s", outpf_path);
           fclose (outpf);
           return (-1);
        }
        if ( fclose (outpf) ) {
           m_sys_err ("Unable to close output (%s)", outpf_path);
           return (-1);
        }
        return (0);
}

int main (void)
{
        if ( init () ) {
           return (1);
        }
        if ( go () ) {
           return (2);
        }
        if ( shut () ) {
           return (3);
        }
        return (0);
}


