#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#define NO_STDERR_OUTP

#define inpf_path               ""
#define outpf_path              ""

/* Messaging functionality: */
#ifdef  NO_STDERR_OUTP

void m_sys_err (const char *format, ...)
{
         if ( format ) {
            /* We don't need "format". */
         }
}

void m_err (const char *format, ...)
{
        if ( format ) {
           /* We don't need "format". */
        }
}

#else /* NO_STDERR_OUTP */

void m_sys_err (const char *format, ...)
{
         va_list                 va;
         int                     o_errno;

         o_errno = errno;
         va_start (va, format);
         fprintf (stderr, "Error:\n\t");
         vfprintf (stderr, format, va);
         fprintf (stderr, "\n\tLast system error: %i %s.\n",
                          o_errno, strerror (o_errno));
         va_end (va);
}

void m_err (const char *format, ...)
{
        va_list                 va;

        va_start (va, format);
        fprintf (stderr, "Error:\n\t");
        vfprintf (stderr, format, va);
        fprintf (stderr, "\n");
        va_end (va);
}

#endif /* NO_STDERR_OUTP */

/* Solution: */
#define LNUM_DGTS_C_MAX         2000
#define LNUM_BASE               1000000

struct path_lm_ts {
       long double              sum,
                                used_sum;
       long double              dividents,
                                u;
};

struct LNum_ts {
       size_t                   dgts_c;
       unsigned long int        dgts [LNUM_DGTS_C_MAX];
};

typedef struct LNum_ts          LNum;

typedef struct path_lm_ts       path_lm_t;

unsigned int                    t;
long double                     p;
long double                     x_0;
long double                     ppcnt,
                                p_k;

long double                     best_sum;
long double                     u_step;
unsigned int                    period_n;
path_lm_t                       sol_path [1000];

#define MAX(a,b)                ( (a) > (b) ? (a) : (b) )

/* Large numbers functionality: */
/*
void lnum_strip_leading_0s (LNum *num)
{
        unsigned long int       i;

        i = num->dgts [c];
        while ( i > 0 ) {
              i--;
              if ( i->dgts [i] != 0 ) {
                 num->dgts_c = i + 1;
                 return;
              }
        }
}

void lnum_set_from_luint (LNum *num, unsigned long int value)
{
        num->dgts_c = 1;
        num->dgts [0] = value;
}

void lnum_add (LNum *num, LNum *addend)
{
        unsigned long int       i;
        unsigned long long int  dgt, carry;

        num->dgts_c = MAX (num->dgts_c, addend->dgts_c) + 1;
        for ( i = 0; i < num->dgts_c; i++ ) {
            dgt = carry + num->dgts [i] + addend [i];
            num->dgts [i] = dgt % LNUM_BASE;
            carry = dgt / LNUM_BASE;
        }
        lnum_strip_leading_0s (num);
}

void lnum_sub (LNum *num, LNum *substr)
{
        unsigned long int       i;
        signed long long int    dgt, carry;

        num->dgts_c = MAX (num->dgts_c, addend->dgts_c);
        for ( i = 0; i < num->dgts_c; i++ ) {
            dgt = carry + num->dgts [i] - substr [i];
            if ( dgt < 0 ) {
               carry = -1;
               dgt += LNUM_BASE;
            }
            num->dgts [i] = dgt;
        }
        lnum_strip_leading_0s (num);
}

void lnum_mul_by_dgt (LNum *num, unsigned long int factor)
{
        unsigned long long int  dgt;
        unsigned long int       carry;
        unsigned long int       i;

        carry = 0;
        for ( i = 0; i < num->dgts_c; i++ ) {
            dgt = num->dgts [i]*factor + carry;
            num->dgts [i] = dgt % LNUM_BASE;
            carry = dgt / LNUM_BASE;
        }
}

void lnum_mul (LNum *res, LNum *a, LNum *b)
{
        LNum                    tmp;
        unsigned long int       i;

        lnum_set_from_luint (res, 0);
        for ( i = 0; i < a->dgts_c; i++ ) {
            luint_set_from_luint (&tmp, 0);
            luint_add (&tmp, a);
            luint_mul_by_dgt (&tmp, b->dgts [i]);
            luint_add (res, &tmp);
        }
}

void lnum_div_by_dgt (LNum *num, unsigned long int divisor)
{
        unsigned long long int   dgt;
        unsigned long int        carry;
        unsigned long int        i;

        carry = 0;
        i = num->dgts_c;
        while ( i > 0 ) {
              i--;
              dgt = (carry * LNUM_BASE + num [i])/divisor;
              num [i] = dgt / LNUM_BASE;
              carry = dgt % LNUM_BASE;
        }
        lnum_strip_leading_0s (num);
}
*/

/*
void print_sol (void)
{
        unsigned int            c;

        printf ("Solution %.6lg:\n", best_sum);
        for ( c = 0; c < period_n; c++ ) {
            printf ("\tsum: %7.6lg, used_sum: %7.6lg, dividents: %7.6lg, "
                   "u: %7.6lg\n",
                   sol_path [c].sum, sol_path [c].used_sum,
                   sol_path [c].dividents, sol_path [c].u);
        }
}

void btrack (long double sum, long double used_sum)
{
        long double               dividents, u;

        if ( period_n >= t ) {
           if ( used_sum > best_sum ) {
              best_sum = used_sum;
              print_sol ();
           }
           return;
        }
        dividents = ppcnt*sum;
        for ( u = 0.0; u <= dividents; u += u_step ) {
            sol_path [period_n].sum = sum;
            sol_path [period_n].used_sum = used_sum;
            sol_path [period_n].dividents = dividents;
            sol_path [period_n].u = u;
            period_n++;
            btrack (sum + dividents - u, used_sum + u);
            period_n--;
        }
}

long double my_pow (long double x, signed int y)
{
        long double           res;
        signed int            c;

        res = 1.0;
        for ( c = 1; c <= y; c++ ) {
            res *= x;
        }
        return (res);
}

void greedy_sol (long double sum, long double used_sum)
{
        long double                  dividents;

        if ( period_n >= t ) {
           if ( used_sum > best_sum ) {
              best_sum = used_sum;
              print_sol ();
           }
           return;
        }
        dividents = ppcnt*sum;
        sol_path [period_n].sum = sum;
        sol_path [period_n].used_sum = used_sum;
        sol_path [period_n].dividents = dividents;
        period_n++;
        if ( (long double) (t - period_n) < my_pow (p_k, t - period_n - 1) ) {
            sol_path [period_n].u = dividents;
            greedy_sol (sum, used_sum + dividents );
        } else {
            sol_path [period_n].u = 0.0;
            greedy_sol (sum + dividents, used_sum);
        }
        period_n--;
}
*/

/* Old heuristic solution: */
/*
void hs_tpds_gen_sol (unsigned int prd, long double sum, long double u)
{
        unsigned int            i;
        long double             csum;

        csum = x_0;
        for ( i = 0; i < prd; i++ ) {
            sol_path [i].sum = csum;
            sol_path [i].used_sum = 0.0;
            sol_path [i].dividents = csum * ppcnt;
            sol_path [i].u = 0.0;
        }
        sol_path [prd].sum = sum;
        sol_path [prd].used_sum = u;
        sol_path [prd].dividents = sum * ppcnt;
        sol_path [prd].u = u;
        for ( i = prd + 1; i < t; i++ ) {
            sol_path [i].sum = sum;
            sol_path [i].used_sum += sum * ppcnt;
            sol_path [i].dividents = sum * ppcnt;
            sol_path [i].u = sum * ppcnt;
        }
}
*/

long double
hs_tpds_used_sum
(unsigned int prd, long double sum, long double u)
{
        return (u + sum * (long double) (t - prd) * ppcnt);
}

void hs_try_prd_split (unsigned prd, long double p_k_pwr)
{
        long double                  sum, dividents, eps;
        long double                  u, u_hi, u_lo, u_mid;
        long double                  usum, usum_lo, usum_hi, usum_mid;

        eps = 1e-8;
        sum = (p_k_pwr/p_k) * x_0;
        dividents = sum * ppcnt;
        u_lo = 0;
        u_hi = dividents;
        u_mid = (u_hi + u_lo)/2.0;
        usum_lo = hs_tpds_used_sum (prd + 1, sum + dividents - u_lo, u_lo);
        usum_hi = hs_tpds_used_sum (prd + 1, sum + dividents - u_hi, u_hi);
        usum_mid = hs_tpds_used_sum (prd + 1, sum + dividents - u_mid, u_mid);
        while ( u_hi - u_lo > eps ) {
              u = (u_lo + u_mid)/2.0;
              usum = hs_tpds_used_sum (prd + 1, sum + dividents - u, u);
              if ( usum_lo < usum && usum < usum_mid ) {
                 u_hi = u_mid;
                 usum_hi = usum_mid;
                 u_mid = (u_hi + u_lo)/2.0;
                 usum_mid = hs_tpds_used_sum (
                            prd + 1, sum + dividents - u_mid, u_mid);
              } else {
                 u_lo = u_mid;
                 usum_lo = usum_mid;
                 u_mid = (u_hi + u_lo)/2.0;
                 usum_mid = hs_tpds_used_sum (
                            prd + 1, sum + dividents - u_mid, u_mid);
              }
        }
        u = (u_lo + u_mid)/2.0;
        usum = hs_tpds_used_sum (prd + 1, sum + dividents - u, u);
        if ( usum > best_sum ) {
           best_sum = usum;
        }
}

void heuristic_sol (void)
{
        unsigned int            i, sol_i;
        long double             sol, p_k_pwr, sol_p_k_pwr;

        p_k_pwr = 1.0;
        for ( i = 0; i < t; i++ ) {
            sol = p_k_pwr * x_0 * (long double)(t - i) * ppcnt;
            p_k_pwr *= p_k;
            if ( sol > best_sum ) {
               best_sum = sol;
               sol_i = i;
               sol_p_k_pwr = p_k_pwr;
            }
        }
        if ( sol_i > 0 ) {
           hs_try_prd_split (sol_i - 1, sol_p_k_pwr/p_k);
        }
        hs_try_prd_split (sol_i, sol_p_k_pwr);
        if ( sol_i + 1 < t ) {
                hs_try_prd_split (sol_i + 1, sol_p_k_pwr*p_k);
        }
}


int go (void)
{
        /*
        printf ("Back-track:\n");
        best_sum = 0.0;
        u_step = 0.1;
        period_n = 0;
        btrack (x_0, 0.0);
        */
        /*
        printf ("Greedy:\n");
        best_sum = 0.0;
        u_step = 0.1;
        period_n = 0;
        greedy_sol (x_0, 0.0);
        */
        /*
        printf ("Heuristic solution:\n");
        */
        best_sum = 0.0;
        heuristic_sol ();
        /*
        printf ("Predicted solution: %7.6g or %7.6g\n",
               (double)t * x_0 * ppcnt,
               my_pow (p_k, t-1) * x_0 * ppcnt);
        */
        return (0);
}

int init (void)
{
        FILE                    *inpf;
        double                  tmp_p, tmp_x_0;

        if ( inpf_path [0] == '\0' ) {
           inpf = stdin;
        } else {
           if ( (inpf = fopen (inpf_path, "r")) == NULL ) {
                  m_sys_err ("Unable to open input (%s)", inpf_path);
                  return (-1);
           }
        }
        if ( fscanf (inpf, "%u", &t) != 1 ||
             fscanf (inpf, "%lf", &tmp_p) != 1 ||
             fscanf (inpf, "%lf", &tmp_x_0) != 1
        ) {
           m_sys_err ("Error parsing input from %s", inpf_path);
           fclose (inpf);
           return (-1);
        }
        p = (long double) tmp_p;
        x_0 = (long double) tmp_x_0;
        p_k = 1.0 + p/100.0;
        ppcnt = p/100.0;
        if ( fclose (inpf) ) {
           m_sys_err ("Unable to close input (%s)", inpf_path);
           return (-1);
        }
        return (0);
}

int shut (void)
{
        FILE                    *outpf;

        if ( outpf_path [0] == '\0' ) {
           outpf = stdout;
        } else {
                if ( (outpf = fopen (outpf_path, "w")) == NULL ) {
                   m_sys_err ("Unable to open output (%s)", outpf_path);
                   return (-1);
                }
        }
        fprintf (outpf, "%.6llf\n", best_sum);
        if ( ferror (outpf) ) {
           m_sys_err ("Error writing output to %s", outpf_path);
           fclose (outpf);
           return (-1);
        }
        if ( fclose (outpf) ) {
           m_sys_err ("Unable to close output (%s)", outpf_path);
           return (-1);
        }
        return (0);
}

int main (void)
{
        if ( init () ) {
           return (1);
        }
        if ( go () ) {
           return (2);
        }
        if ( shut () ) {
           return (3);
        }
        return (0);
}


