#include <iostream>
#include <vector>
#include <ctime>
#include <stdio.h>
#include <string.h>

#define MOD 1000000007LL
#define MAX_LETTERS 26
#define MAX_DISTANCE 5
#define SIZE (MAX_LETTERS * MAX_DISTANCE + 1)

#define DEBUG_INFO 0
using namespace std;

void matrixMul(long long a[SIZE][SIZE], long long b[SIZE][SIZE], long long ans[SIZE][SIZE])
{
	long long ret[SIZE][SIZE];
	memset(ret, 0, sizeof(ret));

	for (int i=0; i<SIZE; i++)
	{
		for (int c=0; c<SIZE; c++)
		{
			for (int j=0; j<SIZE; j++)
			{
				ret[i][c] += a[i][j] * b[j][c];
				if (ret[i][c] >= MOD) ret[i][c] %= MOD;
			}
		}
	}
	memcpy(ans, ret, sizeof(ret));
	return;
}

void matrixPow(long long ma3x[SIZE][SIZE], int power, long long ans[SIZE][SIZE])
{
	long long ret[SIZE][SIZE];
	memset(ret, 0, sizeof(ret));
	for (int i=0; i<SIZE; i++) ret[i][i] = 1;

	while (power)
	{
		if (power & 1) matrixMul(ret, ma3x, ret);
		matrixMul(ma3x, ma3x, ma3x); power >>= 1;
	}
	memcpy(ans, ret, sizeof(ret));
	return;
}

int main(void)
{
	unsigned sTime, eTime;
	FILE *in = stdin; FILE *out = stdout;

	if (DEBUG_INFO)
	{
		sTime = clock();
		in = fopen("reading.in", "rt"); out = fopen("reading.out", "wt");
	}

	// Create empty graph
	long long graph[SIZE][SIZE];
	memset(graph, 0, sizeof(graph));

	// For each letter
	for (int i=0; i<MAX_LETTERS; i++)
	{
		// Add edges to other letters
		for (int c=0; c<MAX_LETTERS; c++) graph[i * MAX_DISTANCE][c * MAX_DISTANCE] = 1;
		// Add edges to self expansions
		for (int c=0; c<MAX_DISTANCE-1; c++) graph[i * MAX_DISTANCE + c + 1][i * MAX_DISTANCE + c] = 1;
	}

	// Add artificial letter to the alphabet (blank)
	//  -- add node to each letter
	for (int i=0; i<MAX_LETTERS; i++)
		graph[MAX_LETTERS * MAX_DISTANCE][i * MAX_DISTANCE] = 1;
	// -- add node to itself (this shortenes the word by one letter)
	graph[MAX_LETTERS * MAX_DISTANCE][MAX_LETTERS * MAX_DISTANCE] = 1;

	// Read input
	int n, m;
	fscanf(in, "%d %d", &n, &m);
	for (int i=0; i<m; i++)
	{
		int distance;
		char let1[4], let2[4];
		fscanf(in, "%s %s %d", let1, let2, &distance);

		// Modify graph to consider new prices
		int idx1 = (int)(let1[0] - 'a') * MAX_DISTANCE;
		int idx2 = (int)(let2[0] - 'a') * MAX_DISTANCE;
		// Remove old edge with weight 1 and add new edge with weight distance
		graph[idx1][idx2] = 0; graph[idx1][idx2 + distance - 1] = 1;
		graph[idx2][idx1] = 0; graph[idx2][idx1 + distance - 1] = 1;
	}
	// Raise graph matrix to the n-th power
	matrixPow(graph, n + 1, graph);

	// The answer is stored in the graph[ARTIFICIAL][EACH_OTHER_LETTER] edges
	int ans = 0;
	for (int i=0; i<MAX_LETTERS; i++)
		ans = (ans + graph[MAX_LETTERS * MAX_DISTANCE][i * MAX_DISTANCE]) % MOD;

	fprintf(out, "%d\n", ans);
	fclose(in); fclose(out);

	if (DEBUG_INFO)
	{
		eTime = clock();
		cout << "Execution time: " << (double)(eTime - sTime) / (double)CLOCKS_PER_SEC << endl;
	}
	return 0;
}
