#include <cstdio>
#include <vector>
#include <cmath>
#include <ctime>

#define F(i,n) for (int i=0; i<n; i++)
#define sqr(x) ((x)*(x))

struct airport		// structure for airports
{
    double x,y,z;
    int refuel;
};

// global variables


const int MAXN = 1001;		// maximum number of airports
const int MAXC = 1001;		// maximum fuel tank capacity
const double EPS = 1e-10;	// epsilon for double arithmetic

int N,M,S,T,K;		// airports count, flights count, source airport, target airport, refueling airports count
double V;		// plane speed
double RR,R;		// squared radius, radius
int C;			// fuel tank capacity

airport a[MAXN];		// airports
std::vector<int> adj[MAXN];	// adjacency lists a.k.a. flights
double dist[MAXN][MAXN];		// matrix with length of each flight
int fuel[MAXN][MAXN];		// matrix with fuel consumption of each flight

double arclength_cos(const airport &A,const airport &B)		// calculate distance of flight using law of cosines
{
    double AB = sqr(A.x-B.x) + sqr(A.y-B.y) + sqr(A.z-B.z);
    double cosa = 1. - AB / (2.*RR);
    return R*acos(cosa);
}


// global functions


void readInput()
{
    scanf("%d %d %lf %d",&N,&M,&V,&C);
    for (int i=1; i<=N ; i++)
     {
            scanf("%lf %lf %lf %d",&a[i].x,&a[i].y,&a[i].z,&a[i].refuel);
            RR = sqr(a[i].x) + sqr(a[i].y) + sqr(a[i].z);
     }
    RR = round(RR);
    R = round(sqrt(RR));
    F(i,M)
     {
        int v,u,f;
        scanf("%d %d %d",&v,&u,&f);
        fuel[v][u] = fuel[u][v] = f;
	    dist[v][u] = dist[u][v] = arclength_cos(a[u],a[v]);

double arclength_cos(const airport &A,const airport &B)		// calculate distance of flight using law of cosines
{
    double AB = sqr(A.x-B.x) + sqr(A.y-B.y) + sqr(A.z-B.z);
    double cosa = 1. - AB / (2.*RR);
    return R*acos(cosa);
}


// global functions


void readInput()
{
    scanf("%d %d %lf %d",&N,&M,&V,&C);
    for (int i=1; i<=N ; i++)
     {
            scanf("%lf %lf %lf %d",&a[i].x,&a[i].y,&a[i].z,&a[i].refuel);
            RR = sqr(a[i].x) + sqr(a[i].y) + sqr(a[i].z);
     }
    RR = round(RR);
    R = round(sqrt(RR));
    F(i,M)
     {
        int v,u,f;
        scanf("%d %d %d",&v,&u,&f);
        fuel[v][u] = fuel[u][v] = f;
        assert(f > 0);
	    dist[v][u] = dist[u][v] = arclength_cos(a[u],a[v]);
        assert(dist[u][v] > 1e-4);
        adj[v].push_back(u); adj[u].push_back(v);
     }
    scanf("%d %d",&S,&T);
    fprintf(stderr," N = %d, M = %d, V = %.3lf, C = %d, R = %.20lf, S = %d, T = %d\n",N,M,V,C,R,S,T);
}


void printOutput(double mindist)
{
    if (mindist > 1e99)
	  printf("0\n");
    else
	  printf("%.10lf\n",mindist/V);
	fprintf(stderr,"Runtime: %.6lf\n",(double)clock()/CLOCKS_PER_SEC);
}        assert(dist[u][v] > 1e-4);
        adj[v].push_back(u); adj[u].push_back(v);
     }
    scanf("%d %d",&S,&T);
    fprintf(stderr," N = %d, M = %d, V = %.3lf, C = %d, R = %.20lf, S = %d, T = %d\n",N,M,V,C,R,S,T);
}


void printOutput(double mindist)
{
    if (mindist > 1e99)
	  printf("0\n");
    else
	  printf("%.10lf\n",mindist/V);
	fprintf(stderr,"Runtime: %.6lf\n",(double)clock()/CLOCKS_PER_SEC);
}

double ans;
int maxdepth = 0;
const int TLE = (int) ((double)CLOCKS_PER_SEC* 4.9);		// time limit set according to problem statement


void bruteforce(int v,int f,double d,int depth,int refuels)
{
    if (clock() > TLE) return;

    if ( v == T )
     {
         if ( d < ans )
	      ans = d;
         return;
     }
    if (depth > maxdepth) return;

    if ( a[v].refuel ) f = C, refuels ++;

    F(i,adj[v].size())
     {
        int u = adj[v][i];
        if (fuel[v][u] > f) continue;
        bruteforce(u, f - fuel[v][u] , d + dist[v][u] , depth + 1 , refuels);
	    if (clock() > TLE) return;
     }
}



int main()
{
    readInput();

    ans=1e100;
    maxdepth = 15;
    for (maxdepth = 2; clock()<TLE; maxdepth ++)
       bruteforce(S,0,0,0,-1);

    printOutput(ans);

    return 0;
}
