#include <cstdio>
#include <vector>
#include <cmath>
#include <stdio.h>
#include <string.h>
#include <algorithm>
#include <queue>

#define F(i,n) for (int i=0; i<n; i++)
#define sqr(x) ((x)*(x))


struct airport		// structure for airports
{
    double x,y,z;
    int refuel;
};


// global variables


const int MAXN = 1001;		// maximum number of airports
const int MAXC = 1001;		// maximum fuel tank capacity
const double EPS = 1e-10;	// epsilon for double arithmetic

int N,M,S,T,K;		// airports count, flights count, source airport, target airport, refueling airports count
double V;		// plane speed
double RR,R;		// squared radius, radius
int C;			// fuel tank capacity

airport a[MAXN];		// airports
std::vector<int> adj[MAXN];	// adjacency lists a.k.a. flights
double dist[MAXN][MAXN];		// matrix with length of each flight
int fuel[MAXN][MAXN];		// matrix with fuel consumption of each flight


double arclength_cos(const airport &A,const airport &B)		// calculate distance of flight using law of cosines
{
    double AB = sqr(A.x-B.x) + sqr(A.y-B.y) + sqr(A.z-B.z);
    double cosa = 1. - AB / (2.*RR);
    return R*acos(cosa);
}


// global functions


void readInput()
{
    scanf("%d %d %lf %d",&N,&M,&V,&C);
    for (int i=1; i<=N ; i++)
     {
            scanf("%lf %lf %lf %d",&a[i].x,&a[i].y,&a[i].z,&a[i].refuel);
            RR = sqr(a[i].x) + sqr(a[i].y) + sqr(a[i].z);
     }
    RR = round(RR);
    R = round(sqrt(RR));
    F(i,M)
     {
        int v,u,f;
        scanf("%d %d %d",&v,&u,&f);
        fuel[v][u] = fuel[u][v] = f;
       // assert(f > 0);
	    dist[v][u] = dist[u][v] = arclength_cos(a[u],a[v]);
       // assert(dist[u][v] > 1e-4);
        adj[v].push_back(u); adj[u].push_back(v);
     }
    scanf("%d %d",&S,&T);
    fprintf(stderr," N = %d, M = %d, V = %.3lf, C = %d, R = %.20lf, S = %d, T = %d\n",N,M,V,C,R,S,T);
}


void printOutput(double mindist)
{
    if (mindist > 1e99)
	  printf("0\n");
    else
	  printf("%.5lf\n",mindist/V);
}

struct state		// structure to store nodes in priority queue
{
    int id;
    double dist;
    int fuel;
    state()
     {
            id = S; dist=0; fuel=0;
     }
    bool operator<(const state &b) const
     {
            return dist > b.dist;				// reverse "<" operator for STL priority queue compatibility
     }
};


bool used[MAXN][MAXC];
double mindist[MAXN][MAXC];


double dijkstra()
{
    for (int i=1; i<=N ;i++)
     F(f,C+1)
      mindist[i][f] = 1e100;
    mindist[S][0] = 0;

    std::priority_queue<state> q;
    q.push( state() );
    state cur,next;

    while ( !q.empty() )
     {
	  cur = q.top();
	  q.pop();
	  if ( used[cur.id][cur.fuel] ) continue;

	  used[cur.id][cur.fuel] = 1;

	  F(i,adj[cur.id].size())
	   {
            next.id = adj[cur.id][i];
            next.dist = cur.dist + dist[cur.id][next.id];
            next.fuel = cur.fuel + fuel[cur.id][next.id];

		    if ( next.fuel <= C && next.dist < mindist[next.id][next.fuel] )
		     {
			     if ( a[next.id].refuel ) next.fuel = 0;
			     q.push(next);
			     mindist[next.id][next.fuel] = next.dist;
		     }
	   }
     }

    double ret = 1e100;
    F(i,C+1)
     if (ret > mindist[T][i])
      ret = mindist[T][i];
    return ret;
}

int main()
{
    readInput();

    printOutput(dijkstra());
}

