/*
 * Author: Cosmin Gheorghe
 * Time Complexity: O(N^3)
 */

#include <stdio.h>
#include <math.h>
#include <vector>
#include <algorithm>
using namespace std;

#define MP make_pair
#define ff first
#define ss second
#define eps 1e-8

#define NMAX 1005
const double pi = acos(-1);

int N;

double xp[NMAX], yp[NMAX];

vector <pair<double, double> > C;

double din[NMAX];

pair<double, double> pct[NMAX];

double rmin[NMAX][NMAX];

void baga_cerc(double x, double y)
{
	C.push_back(MP(x, y));
}

void baga_cerc(double x1, double y1, double x2, double y2)
{
	if (x1 == x2) return;

	double xm = (x1 + x2) * 0.5;
	double ym = (y1 + y2) * 0.5;
	double ap = x1 - x2;
	double bp = y1 - y2;

	double xf = (ap * xm + bp * ym) / ap;
	double rf = sqrt((xf - x1) * (xf - x1) + y1 * y1);

	C.push_back(MP(xf, rf));
}

int in_cerc(double x, double y, double c, double r)
{
	return (x - c) * (x - c) + y * y <= r * r + eps;
}

int main()
{
	int i, j;

	freopen("acerc.in", "r", stdin);
	freopen("acerc.out", "w", stdout);

	scanf("%d", &N);

	for (i = 1; i <= N; i++) {
		scanf("%lf %lf", &xp[i], &yp[i]);
		pct[i] = MP(xp[i], yp[i]);
	}

	sort(pct + 1, pct + N + 1);

	for (i = 1; i <= N; i++) xp[i] = pct[i].ff, yp[i] = pct[i].ss;

	for (i = 1; i <= N; i++) {
		baga_cerc(xp[i], yp[i]);
		for (j = i + 1; j <= N; j++) {
			baga_cerc(xp[i], yp[i], xp[j], yp[j]);
		}
	}

	
	int n = C.size();

	for (i = 1; i <= N; i++)
		for (j = 1; j <= N; j++)
			rmin[i][j] = 1e+10;

	for (i = 0; i < n; i++) {
		double xc = C[i].ff, rc = C[i].ss;

		int st = 0, dr = N + 1;

		for (j = 1; j <= N; j++) {
			if (in_cerc(xp[j], yp[j], xc, rc)) continue;

			if (xp[j] <= xc) st = max(st, j);
			if (xp[j] >= xc) dr = min(dr, j);
		}
	
		if (st+1 <= dr-1) rmin[st+1][dr-1] = min(rmin[st+1][dr-1], rc * rc);
	}

	for (int len = N; len >= 1; len--) {
		for (i = 1; i + len - 1 <= N; i++) {
			j = i + len - 1;

			rmin[i+1][j] = min(rmin[i+1][j], rmin[i][j]);
			rmin[i][j-1] = min(rmin[i][j-1], rmin[i][j]);
		}
	}

	din[0] = 0;

	for (i = 1; i <= N; i++) {
		din[i] = 1e+10;

		for (j = 1; j <= i; j++) {
			din[i] = min(din[i], din[j-1] + rmin[j][i]);
		}
	}

	printf("%0.4lf\n", din[N] * pi);

	return 0;
}

