#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <vector>
#include <map>
#include <algorithm>

using namespace std;

#define NMAX 40
#define INF 1e12
#define eps 1e-5
#define bsrc_eps 1e-7

#define vmin(a,b) ((a) <= (b) ? (a) : (b))
#define vmax(a,b) ((a) >= (b) ? (a) : (b))

vector<pair<double, int> > vdist;
double minRadius[1 << ((NMAX / 2) + 1)], optSol[1 << ((NMAX / 2) + 1)];
long long bit[NMAX], subset, subsetBig, s1, s2, lim;
double x[NMAX], y[NMAX], xp[NMAX], yp[NMAX], xmin, xmax, rmin, ymax, R, xc, aa, bb, xm, ym, am, bm, cm, aox, box, cox, dist, minSum, li, ls, xleft, xright;
char inside[NMAX];
int i, j, k, N, q, idx, idxBig, idxDif;

void genSubsets(int p, int pmin, int pmax)
{
	if (p >= pmax)
	{
		if (subset == 0)
		{
			if (pmax == N)
				idx = lim;
			else
				idx = 0;

			minRadius[idx] = 0.0;
			optSol[idx] = 0.0;
			return;
		}

		// compute the minimum radius of a circle with the center on the OX axis which contains all the points in the subset 'subset'
		rmin = INF;
		ymax = -1.0;
		
		for (i = pmin; i < pmax; i++)
			if ((subset & bit[i]) && (y[i] > ymax))
				ymax = y[i];

		li = ymax * ymax; ls = INF;

		while (fabs(ls - li) > bsrc_eps)
		{
			R = 0.5 * (li + ls);

			xmin = INF;
			xmax = -INF;

			for (i = pmin; i < pmax; i++)
				if (subset & bit[i])
				{
					xleft = x[i] - sqrt(R - y[i] * y[i]);
					xright = x[i] + sqrt(R - y[i] * y[i]);

					xmin = min(xmin, xright);
					xmax = max(xmax, xleft);
				}

			if (xmax < xmin || fabs(xmax - xmin) < eps)
			{
				rmin = ls = R;
			}
			else
				li = R;
		}

		if (pmax == N)
			idx = lim + (subset >> (N / 2));
		else
			idx = (int) subset;

		minRadius[idx] = rmin;
		optSol[idx] = rmin;
	}
	else
	{
		subset |= bit[p];
		genSubsets(p + 1, pmin, pmax);
		subset ^= bit[p];
		genSubsets(p + 1, pmin, pmax);
	}
}

void DP(int p, int pmax)
{
	if (p >= pmax)
	{
		if (pmax == N)
		{
			idx = (int) (lim + (subset >> (N / 2)));
			idxBig = (int) (lim + (subsetBig >> (N / 2)));
			idxDif = (int) lim + (idxBig ^idx);
		}
		else
		{
			idx = (int) subset;
			idxBig = (int) subsetBig;
			idxDif = (int) (subsetBig ^ subset);
		}

		optSol[idxBig] = vmin(optSol[idxBig], optSol[idxDif] + minRadius[idx]);
	}
	else
	{
		DP(p + 1, pmax);

		subsetBig |= bit[p];
		DP(p + 1, pmax);

		subset |= bit[p];
		DP(p + 1, pmax);

		subsetBig ^= bit[p];
		subset ^= bit[p];
	}
}

int main()
{
	int tstart = clock();

	freopen("acerc.in", "r", stdin);
	scanf("%d", &N);
	for (i = 0; i < N; i++)
	{
		scanf("%lf %lf", &x[i], &y[i]);
		if (y[i] < 0)
			y[i] = -y[i];

		if (i == 0)
			xmin = x[i];
		else
			xmin = vmin(xmin, x[i]);
	}
	
	for (i = 0; i < N; i++)
		x[i] -= xmin;

	for (i = 0; i < N; i++)
		vdist.push_back(make_pair(x[i] * x[i] + y[i] * y[i], i));

	sort(vdist.begin(), vdist.end());

	for (i = 0; i < N; i++)
	{
		xp[i] = x[vdist[i].second];
		yp[i] = y[vdist[i].second];
	}

	for (i = 0; i < N; i++)
	{
		x[i] = xp[i];
		y[i] = yp[i];
	}

	bit[0] = 1;
	for (i = 1; i < NMAX; i++)
		bit[i] = bit[i - 1] * 2;

	lim = bit[N / 2];

	subset = 0;
	genSubsets(0, 0, N / 2);

	subset = subsetBig = 0;
	DP(0, N / 2);

	subset = 0;
	genSubsets(N / 2, N / 2, N);

	subset = subsetBig = 0;
	DP(N / 2, N);

	// try all possibilities for the middle circle
	s1 = 0;
	for (i = 0; i < N / 2; i++)
		s1 |= bit[i];

	s2 = 0;
	for (i = N / 2; i < N; i++)
		s2 |= bit[i];

	minSum = optSol[s1] + optSol[lim + (s2 >> (N / 2))];

	for (i = 0; i < N; i++)
	{
		R = y[i] * y[i];
		xc = x[i];

		memset(inside, 0, sizeof(inside));
		for (k = 0; k < N; k++)
		{
			dist = (x[k] - xc) * (x[k] - xc) + y[k] * y[k];
			if (dist < R || fabs(dist - R) < eps)
				inside[k] = 1;
		}

		s1 = 0;
		for (k = 0; k < N / 2; k++)
			if (!inside[k])
				s1 |= bit[k];

		s2 = 0;
		for (k = N / 2; k < N; k++)
			if (!inside[k])
				s2 |= bit[k];

		minSum = vmin(minSum, R + optSol[s1] + optSol[lim + (s2 >> (N / 2))]);
	}

	for (i = 0; i < N; i++)
		for (j = i + 1; j < N; j++)
		{
			aa = y[i] - y[j];
			bb = x[j] - x[i];
			xm = (x[i] + x[j]) * 0.5;
			ym = (y[i] + y[j]) * 0.5;

			am = -bb;
			bm = aa;
			cm = -am*xm - bm*ym;

			if (fabs(am) < eps)
				continue;

			xc = -cm / am;
			R = (xc - x[i]) * (xc - x[i]) + y[i] * y[i];

			memset(inside, 0, sizeof(inside));
			for (k = 0; k < N; k++)
			{
				dist = (x[k] - xc) * (x[k] - xc) + y[k] * y[k];
				if (dist < R || fabs(dist - R) < eps)
					inside[k] = 1;
			}

			s1 = 0;
			for (k = 0; k < N / 2; k++)
				if (!inside[k])
					s1 |= bit[k];

			s2 = 0;
			for (k = N / 2; k < N; k++)
				if (!inside[k])
					s2 |= bit[k];

			minSum = vmin(minSum, R + optSol[s1] + optSol[lim + (s2 >> (N / 2))]);
		}

	freopen("acerc.out", "w", stdout);
	printf("%.5lf\n", minSum);
	fprintf(stderr, "%.5lf\n", minSum);

	//fprintf(stderr, "Duration=%.3lf msec\n", ((double) (clock() - tstart)) / CLOCKS_PER_SEC);
	return 0;
}
