/*
 * Author: Andrei Grigorean
 * Time complexity: O(N log N)
 * Memory complexity: O(N)
 * Expected score: 100/100 points
 */

#include <cassert>
#include <cstdio>
#include <algorithm>
#include <vector>
using namespace std;

const int MAX_N = 50005;

struct Punct {
    int x, y;

    Punct(int _x = 0, int _y = 0) {
        x = _x;
        y = _y;
    }

    bool operator<(const Punct&) const;

    inline int sgn() const;
};

inline int sign(const Punct &a, const Punct &b, const Punct &c) {
    long long det = 1LL * (a.x - c.x) * (b.y - c.y) - 1LL * (b.x - c.x) * (a.y - c.y);
    if (det < 0) return -1;
    if (det > 0) return 1;
    return 0;
}

int n;
Punct closca;
Punct v[2 * MAX_N];

inline int Punct::sgn() const {
    return y > closca.y || (y == closca.y && x > closca.x) ? 1 : -1;
}

bool Punct::operator<(const Punct& other) const {
    int sgnThis = (*this).sgn();
    int sgnOther = other.sgn();
    if (sgnThis != sgnOther)
        return sgnThis == 1;

    return sign(*this, other, closca) == 1;
}

int main() {
    assert(freopen("hawaii.in", "r", stdin) != NULL);
    assert(freopen("hawaii.out", "w", stdout) != NULL);

    assert(scanf("%d", &n) == 1);
    assert(scanf("%d %d", &closca.x, &closca.y) == 2);
    for (int i = 0; i < n; ++i)
        assert(scanf("%d %d", &v[i].x, &v[i].y) == 2);

    sort(v, v + n);
    for (int i = 0; i < n; ++i)
        v[i + n] = v[i];

    long long sol = 0;
    for (int p1 = 0, p2 = 0; p1 < n; ++p1) {
        while (p2 < p1 + n && sign(v[p1], v[p2], closca) != -1)
            ++p2;
        sol += 1LL * (p2 - p1 - 1) * (p2 - p1 - 2) / 2;
    }
    sol = 1LL * n * (n - 1) * (n - 2) / 6 - sol;

    printf("%lld\n", sol);
}
